## LAPS project data convertion, storage, and plotting based on the .json schema file
# Ekaterina Bolotskaya
# 07/17/2023

def LAPS_1_empty_dir_create_update(schema_path, target_dir):
    """
    Update the directory structure based on the provided JSON schema file.
    If this folder already exists, it will be UPDATED. The folders that exist, but no longer correspond to the .json structure
    will be deleted if empty and kept if not empty.

    Parameters:
    schema_path (str): The path to the JSON schema file that defines the directory structure.
    target_dir (str): The path to the target directory where the directory structure will be created/updated.

    Returns:
    None

    This function reads the JSON schema file, creates directories, and subdirectories based on the schema.
    The function is designed to update the structure, preserving existing data.
    """

    # Directory creation function
    def create_directory(path):
        """
        Create a directory at the given path if it does not already exist.

        Parameters:
        path (str): The path to the directory that needs to be created.

        Returns:
        None
        """
        if not os.path.exists(path):
            os.makedirs(path)

    # Parse the JSON
    with open(schema_path, 'r') as f:
        json_data = json.load(f)

    # Create directories for the first tier of subfields within "object"
    create_directory(target_dir)

    def create_subdirectories(base_dir, path_list):
        """
        Create subdirectories within the base directory based on the provided path list.

        Parameters:
        base_dir (str): The base directory where subdirectories will be created.
        path_list (list): A list of directory names to be created as subdirectories within the base directory.

        Returns:
        None
        """
        current_dir = base_dir
        for subdir_name in path_list:
            current_dir = os.path.join(current_dir, subdir_name)
            create_directory(current_dir)

    def process_json_data(base_dir, data, path_list=[]):
        """
        Recursively process JSON data and create directories as needed.

        Parameters:
        base_dir (str): The base directory.
        data (dict): The JSON data to process.
        path_list (list): A list of directory names representing the path.

        Returns:
        None
        """
        if isinstance(data, dict):
            for key, value in data.items():
                sub_dir = os.path.join(base_dir, key)
                create_directory(sub_dir)
                process_json_data(sub_dir, value, path_list + [key])

                if key == "data" and isinstance(value, dict) and "datasets" in value:
                    datasets_dir = os.path.join(sub_dir, "datasets")
                    create_directory(datasets_dir)

                    if "datasets" in value:
                        for dataset in value["datasets"]:
                            data_value = dataset.get("data", "")
                            dataset_id = dataset.get("id", "")
                            if data_value:
                                if dataset_id:
                                    dataset_subdir = os.path.join(datasets_dir, data_value.replace(" ", "_") + "_" + dataset_id)
                                else:
                                    dataset_subdir = os.path.join(datasets_dir, data_value)
                                create_directory(dataset_subdir)

                                #print(dataset_subdir)

                if "documents" in key:
                    create_subdirectories(base_dir, path_list)  # Create subdirectories leading to "documents"

    process_json_data(target_dir, json_data)

    print("Empty directory created successfully. Please populate it with data for further processing.")
    print(target_dir)